#include "DFRobot_OLED12864.h"
#include "DFRobot_SpeechSynthesis.h"

// Debug 模式开关
#define NCDEBUG 1

// 语音合成模块
DFRobot_SpeechSynthesis_I2C ss;

// OLED 地址
const uint8_t OLED_I2C_addr = 0x3c;
const uint8_t OLED_pin_SPI_cs = 9;

// OLED 对象
DFRobot_OLED12864 OLED(OLED_I2C_addr, OLED_pin_SPI_cs);

// 矩阵开关行列定义
#define KEY_ROW1 27
#define KEY_ROW2 15
#define KEY_ROW3 13
#define KEY_COL1 14
#define KEY_COL2 4
#define KEY_COL3 16
#define KEY_COL4 17

// 菜单向上键
#define UP_KEY 26
// 菜单确认键
#define CONFIRM_KEY 5
// 菜单向下
#define DOWN_KEY 25

// 当前菜单指针
int Menu = 0;

// 当前按键信息
enum KEYPRESSED
{
  NONEPRESSED = 0, UPPRESSED, DOWNPRESSED, CONFIRMPRESSED
};

// 运算符，分别是加减乘除
enum MATHOPERATION
{
  OPADD = 0, OPSUB, OPMUL, OPDIV
};

// 菜单项目总数
#define MENUITEMCOUNTER 7
char MainMenu[MENUITEMCOUNTER][34] =
{
  "个位数相加",
  "个位数相减",
  "个位数相乘",
  "两位数相加",
  "两位数相减",
  "两位数乘一位数",
  "三个数混合运算",
};

// 在 OLED 上绘制菜单
void ShowMainMenu(int Current)
{
  char Buffer[40];

  OLED.clear();
  // 第一个条目前添加  "→" 符号
  sprintf(Buffer, "→%s", MainMenu[Current]);
  OLED.disStr(0, 0, Buffer);
  // 其余条目前添加空格和第一个条目对齐
  sprintf(Buffer, "  %s", MainMenu[(Current + 1) % MENUITEMCOUNTER]);
  OLED.disStr(0, 16, Buffer );
  sprintf(Buffer, "  %s", MainMenu[(Current + 2) % MENUITEMCOUNTER]);
  OLED.disStr(0, 32, Buffer);
  sprintf(Buffer, "  %s", MainMenu[(Current + 3) % MENUITEMCOUNTER]);
  OLED.disStr(0, 48, Buffer);

  OLED.display();
}

// 返回当前按键信息
KEYPRESSED GetKeyPressed()
{
  if (digitalRead(UP_KEY) == LOW)
  {
    delay(200);
    return UPPRESSED;
  }
  if (digitalRead(DOWN_KEY) == LOW)
  {
    delay(200);
    return DOWNPRESSED;
  }
  if (digitalRead(CONFIRM_KEY) == LOW)
  {
    delay(200);
    return CONFIRMPRESSED;
  }
  return NONEPRESSED;
}

// 扫描按钮矩阵，返回当前的输入值
int GetMatrix()
{
  int result = 0;

  digitalWrite(KEY_ROW1, HIGH);
  digitalWrite(KEY_ROW2, LOW);
  digitalWrite(KEY_ROW3, LOW);
  delay(10);

  // 第一行
  result = (digitalRead(KEY_COL1) << 3) +
           (digitalRead(KEY_COL2) << 2) +
           (digitalRead(KEY_COL3) << 1) +
           digitalRead(KEY_COL4) ;
  if (NCDEBUG)
  {
    Serial.print(digitalRead(KEY_COL1) << 3);
    Serial.print(digitalRead(KEY_COL2) << 2);
    Serial.print(digitalRead(KEY_COL3) << 1);
    Serial.print(digitalRead(KEY_COL4));
    Serial.println("");
  }

  // 中间行
  digitalWrite(KEY_ROW1, LOW);
  digitalWrite(KEY_ROW2, HIGH);
  digitalWrite(KEY_ROW3, LOW);
  delay(10);
  result = ((digitalRead(KEY_COL1) << 3) +
            (digitalRead(KEY_COL2) << 2) +
            (digitalRead(KEY_COL3) << 1) +
            (digitalRead(KEY_COL4))) +
           result * 10;
  if (NCDEBUG)
  {
    Serial.print(digitalRead(KEY_COL1));
    Serial.print(digitalRead(KEY_COL2));
    Serial.print(digitalRead(KEY_COL3));
    Serial.print(digitalRead(KEY_COL4));
    Serial.println("");
    Serial.print(">>>>>>");
    Serial.println(result);
  }

  // 最底行
  digitalWrite(KEY_ROW1, LOW);
  digitalWrite(KEY_ROW2, LOW);
  digitalWrite(KEY_ROW3, HIGH);
  delay(10);
  result = (digitalRead(KEY_COL1) << 3) +
           (digitalRead(KEY_COL2) << 2) +
           (digitalRead(KEY_COL3) << 1) +
           (digitalRead(KEY_COL4) << 0) +
           result * 10;
  if (NCDEBUG)
  {
    Serial.print(digitalRead(KEY_COL1));
    Serial.print(digitalRead(KEY_COL2));
    Serial.print(digitalRead(KEY_COL3));
    Serial.print(digitalRead(KEY_COL4));
    Serial.println("");
  }
  if (NCDEBUG)
  {
    Serial.print("GetMatrix Result:");
    Serial.println(result);
  }

  return result;
}

// 语音输出一个数字
void readValue(int Value)
{
  if (NCDEBUG)
  {
    Serial.print("ReadValue:");
    Serial.println(Value);
  }

  String DataBuffer[12] = {{"零"}, {"一"}, {"二"}, {"三"}, {"四"}, {"五"}, {"六"}, {"七"}, {"八"}, {"九"}, {"十"}, {"百"}};

  // 百位不为零
  if (Value / 100 != 0)
  {
    // 读出百位
    ss.speak(DataBuffer[Value / 100]);
    // 读出"百"
    ss.speak(DataBuffer[11]);
    // 如果十位为 0 ,那么直接读出"零"
    if (Value / 10 % 10 == 0)
    {
      ss.speak(DataBuffer[0]);
    }
  }
  else
  {
    // 十位不为零
    if ((Value / 10 % 10) != 0)
    {
      // 读出十位（不含0的情况）
      if (Value / 10 == 1) { // 13 这种直接读“十三”
        ss.speak(DataBuffer[10]);
      } else {  // 23 这种读出“二十三”
        ss.speak(DataBuffer[Value / 10 % 10]);
        ss.speak(DataBuffer[10]);
      }
    }
  }

  // 个位不为零
  if ((Value % 10) != 0)
  {
    ss.speak(DataBuffer[Value % 10]);
  }

  // 零特别处理
  if (Value == 0)
  {
    ss.speak(DataBuffer[0]);
  }
}

// 语音输出表达式
void readOp(MATHOPERATION Op)
{
  if (NCDEBUG)
  {
    Serial.print("Operation:");
    Serial.println((int) Op);
  }
  String DataBuffer[4] = {{"加"}, {"减"}, {"乘"}, {"除"}};
  ss.speak(DataBuffer[(int) Op]);
}

void setup(void)
{
  Serial.begin(115200);

  // 初始化屏幕
  OLED.init();
  OLED.flipScreenVertically();

  // 初始化语音模块
  ss.begin();

  // 初始化开关矩阵
  pinMode(KEY_COL1, INPUT_PULLDOWN);
  pinMode(KEY_COL2, INPUT_PULLDOWN);
  pinMode(KEY_COL3, INPUT_PULLDOWN);
  pinMode(KEY_COL4, INPUT_PULLDOWN);

  pinMode(KEY_ROW1, OUTPUT);
  pinMode(KEY_ROW2, OUTPUT);
  pinMode(KEY_ROW3, OUTPUT);

  // 初始化按键
  pinMode(UP_KEY, INPUT_PULLUP);
  pinMode(CONFIRM_KEY, INPUT_PULLUP);
  pinMode(DOWN_KEY, INPUT_PULLUP);

  ShowMainMenu(Menu);
  ss.speak("夜间数学计算器");

}

byte Status = 1;

// 当前算式运算结果
int Result;
// 参加运算的数字数量
int CalcNum;
// 参加运算的数字
int Num[3];
// 参加运算的运算符
MATHOPERATION Op[2];

void loop(void)
{
  if (Status == 1)
  {
    if (NCDEBUG) {
      Serial.println("Status 1");
    }
    // 显示菜单
    ShowMainMenu(Menu);
    if (NCDEBUG) {
      Serial.println("ShowMenu");
    }
    // 取得按键信息
    KEYPRESSED Key = GetKeyPressed();
    while (Key == NONEPRESSED)
    {
      Key = GetKeyPressed();
    }
    if (Key == UPPRESSED)
    {
      Status = 2;
    }

    if (Key == DOWNPRESSED)
    {
      Status = 3;
    }
    if (Key == CONFIRMPRESSED)
    {
      ss.speak(MainMenu[Menu]);
      Status = 4;
    }
  }
  if (Status == 2)
  {
    Menu = (Menu + MENUITEMCOUNTER - 1) % MENUITEMCOUNTER;
    Status = 1;
  }
  if (Status == 3)
  {
    Menu = (Menu + 1) % MENUITEMCOUNTER;
    Status = 1;
  }

  if (Status == 4)
  {
    // 生成要计算的表达式
    switch (Menu)
    {
      case 0: //"个位数相加"
        CalcNum = 2; // 2个数字运算
        Num[0] = random(10);
        Num[1] = random(10);
        Op[0] = OPADD;
        Result = Num[0] + Num[1];
        break;
      case 1://"个位数相减"
        CalcNum = 2; // 2个数字运算
        do
        {
          Num[0] = random(10);
          Num[1] = random(10);
        }
        while (Num[0] < Num[1]);   //避免结果是负数
        Op[0] = OPSUB;
        Result = Num[0] - Num[1];
        break;
      case 2://"个位数相乘"
        CalcNum = 2; // 2个数字运算
        Num[0] = random(10);
        Num[1] = random(10);
        Op[0] = OPMUL;
        Result = Num[0] * Num[1];
        break;
      case 3://"两位数相加"
        CalcNum = 2; // 2个数字运算
        Num[0] = random(100);
        Num[1] = random(91) + 10;
        Op[0] = OPADD;
        Result = Num[0] + Num[1];
        break;
      case 4://"两位数相减"
        CalcNum = 2; // 2个数字运算
        do
        {
          Num[0] = random(100);
          Num[1] = random(91) + 10;
        }
        while (Num[0] < Num[1]);   //避免结果是负数
        Op[0] = OPSUB;
        Result = Num[0] - Num[1];
        break;
      case 5://"两位数乘一位数"
        CalcNum = 2; // 2个数字运算
        Num[0] = random(90) + 10;
        Num[1] = random(9) + 1;
        Op[0] = OPMUL;
        Result = Num[0] * Num[1];
        break;
      case 6://"三个数混合运算"
        CalcNum = 3; // 3个数字运算
        do
        {
          Num[0] = random(20);
          Num[1] = random(20);
          Num[2] = random(20);
          Op[0] = (MATHOPERATION) random(2); //只生成加减的运算
          Op[1] = (MATHOPERATION) random(2); //只生成加减的运算
          if (Op[0] == OPADD)
          {
            Result = Num[0] + Num[1];
          }
          if (Op[0] == OPSUB)
          {
            Result = Num[0] - Num[1];
          }

          if (Op[1] == OPADD)
          {
            Result = Result + Num[2];
          }
          if (Op[1] == OPSUB)
          {
            Result = Result - Num[2];
          }
          // 检查生成计算式
          //  1.避免结果是负数
          //  2.避免后面的运算符优先级高于前一个
        }
        while ((Result < 0) || ((Op[1] > 1) && (Op[0] < 2)));

        break;
      default:
        {}
    }
    Status = 5;
  }
  if (Status == 5)
  {
    if (NCDEBUG)
    {
      Serial.print("Status 5");
    }
    // 语音播报表达式
    readValue(Num[0]);
    readOp(Op[0]);
    readValue(Num[1]);
    if (NCDEBUG)
    {
      Serial.print(Num[0]);
      Serial.print(" ");
      Serial.print(Num[1]);
    }
    // 如果是三个数运算，那么多读出一个运算符和运算数
    if (CalcNum == 3)
    {
      readOp(Op[1]);
      readValue(Num[2]);
    }

    char Exp[16];
    sprintf(Exp, "%d", Num[0]);
    if (Op[0] == OPADD) {
      strcat(Exp, "+");
    }
    if (Op[0] == OPSUB) {
      strcat(Exp, "-");
    }
    if (Op[0] == OPMUL) {
      strcat(Exp, "*");
    }
    if (Op[0] == OPDIV) {
      strcat(Exp, "/");
    }

    char tmpInt[4];
    itoa(Num[1], tmpInt, 10);
    strcat(Exp, tmpInt);
    // 如果是三个数运算，那么多读出一个运算符和运算数
    if (CalcNum == 3)
    {
      if (Op[1] == OPADD) {
        strcat(Exp, "+");
      }
      if (Op[1] == OPSUB) {
        strcat(Exp, "-");
      }
      if (Op[1] == OPMUL) {
        strcat(Exp, "*");
      }
      if (Op[1] == OPDIV) {
        strcat(Exp, "/");
      }
      itoa(Num[2], tmpInt, 10);
      strcat(Exp, tmpInt);
    }                   
    OLED.disStr(0, 16, "　　　　　　　　");
    OLED.disStr(0, 16, Exp);
    OLED.disStr(0, 32, "上键返回中键提交");
    OLED.disStr(0, 48, "下键重新播报题目");
    OLED.display();

    Status = 6;
  }

  if (Status == 6)
  {
    // 取得按键信息
    KEYPRESSED Key = GetKeyPressed();
    while (Key == NONEPRESSED)
    {
      Key = GetKeyPressed();
    }
    // 向上键返回主菜单
    if (Key == UPPRESSED)
    {
      if (NCDEBUG) {
        Serial.println("Return main menu");
      }
      Status = 1;
    }
    // 向下键重新播报算式
    if (Key == DOWNPRESSED)
    {
      if (NCDEBUG)
      {
        Serial.print(Num[0]);
        Serial.print(" ");
        Serial.print((int) Op[0]);
        Serial.print(" ");
        Serial.println(Num[1]);
      }
      Status = 5;
    }
    // 确认键检查输入
    if (Key == CONFIRMPRESSED)
    {
      if (GetMatrix() == Result)
      {
        // 输入正确
        Status = 8;
      }
      else
      {
        // 输入错误
        Status = 7;
      }
    }
  }
  // 输入错误
  if (Status == 7)
  {
    ss.speak("结果错误");
    // 重新等待输入
    Status = 6;
  }
  // 输入正确
  if (Status == 8)
  {
    ss.speak("结果正确，下一题");
    // 重新出题
    Status = 4;
  }
}




/* 生成表达式测试
    Menu=(Menu+1)%7;
     if (CalcNum==2) {
          Serial.print(Num[0]);
          if (Op[0]==OPADD) {
              Serial.print("+");
            }
          if (Op[0]==OPSUB) {
              Serial.print("-");
            }
          if (Op[0]==OPMUL) {
              Serial.print("*");
            }
          if (Op[0]==OPDIV) {
              Serial.print("+");
            }
          Serial.print(Num[1]);
          Serial.print("=");
          Serial.println(Result);
      }
     if (CalcNum==3) {
          Serial.print(Num[0]);
          if (Op[0]==OPADD) {
              Serial.print("+");
            }
          if (Op[0]==OPSUB) {
              Serial.print("-");
            }
          if (Op[0]==OPMUL) {
              Serial.print("*");
            }
          if (Op[0]==OPDIV) {
              Serial.print("+");
            }
          Serial.print(Num[1]);
          if (Op[1]==OPADD) {
              Serial.print("+");
            }
          if (Op[1]==OPSUB) {
              Serial.print("-");
            }
          if (Op[1]==OPMUL) {
              Serial.print("*");
            }
          if (Op[1]==OPDIV) {
              Serial.print("+");
            }
          Serial.print(Num[2]);
          Serial.print("=");
          Serial.println(Result);
      }
*/
